/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.log.mongodb;

import java.net.UnknownHostException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.ErrorManager;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;
import java.util.logging.StreamHandler;

import org.ow2.petals.commons.log.LogData;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.Mongo;
import com.mongodb.ServerAddress;
import com.mongodb.WriteConcern;

/**
 * A JUL handler which write MONIT logs to mongodb
 * 
 * @author Christophe HAMERLING - Linagora
 */
public class MongoDbHandler extends StreamHandler {

    public static final String HOST  ="host";
    public static final String DEFAULT_HOST  = "localhost";
    public static final String PORT  ="port";
    public static final String DEFAULT_PORT  ="27017";
    public static final String COLLECTION  ="collection";
    public static final String DEFAULT_COLLECTION  ="logs";
    public static final String DB  ="db";
    public static final String DEFAULT_DB  ="petals";
    public static final String LOGIN  ="login";
    public static final String PASSWORD  ="password";
    public static final String LOGGER_LEVEL  = "level";
    public static final String LOGGER_NAME = "name";
    public static final String LOGGER_THREAD_ID = "thread_id";
    public static final String LOGGER_MESSAGE = "message";
    public static final String LOGGER_TIME_MS = "time_ms";

    protected String host;

    protected String port;

    protected String collectionName;

    protected String db;

    protected String login;

    protected String password;

    protected Mongo mongo;

    protected DB mongoDB;

    protected DBCollection collection;

    public MongoDbHandler() {
        super();
    }

    @Override
    public synchronized void publish(final LogRecord record) {
        if (record.getLevel().intValue() == org.ow2.petals.commons.log.Level.MONIT.intValue()) {
            if (null == mongoDB) {
                try {
                    doConnect();
                } catch (UnknownHostException e) {
                    throw new RuntimeException(e.getMessage(), e);
                }
            }
            this.pushLog(record);
        }
    }

    private boolean isThereLogDataParameter(Object[] logParameters) {
        return (logParameters != null)
                && (logParameters.length != 0)
                && logParameters[0] != null
                && ReflectionHelper.isOrInheritedFrom(logParameters[0].getClass(),
                        LogData.class.getCanonicalName());
    }

    /**
     * Write log record to collection
     *
     * @param record
     */
    private void pushLog(final LogRecord record) {
        try {
            DBObject dbo = new BasicDBObject();
            dbo.put(LOGGER_LEVEL, record.getLevel().getName());
            dbo.put(LOGGER_NAME, record.getLoggerName());
            dbo.put(LOGGER_THREAD_ID, record.getThreadID());
            dbo.put(LOGGER_TIME_MS, record.getMillis());
            
            final Map<String, Object> extraParameters = new LinkedHashMap<String, Object>();
            final Object[] logExtraParameters = record.getParameters();
            if (isThereLogDataParameter(logExtraParameters)) {
                extraParameters.putAll((Map<String, Object>) logExtraParameters[0]);
            }
            
            final Iterator<Entry<String, Object>> itExtraParameters = extraParameters.entrySet().iterator();
            while (itExtraParameters.hasNext()) {
                final Entry<String, Object> entry = itExtraParameters.next();
                final Object value = entry.getValue();
                if (value == null) {
                    dbo.put(entry.getKey(), "");
                }
                else {
                    dbo.put(entry.getKey(), value.toString());
                }
            }

            dbo.put(LOGGER_MESSAGE, record.getMessage());
            
            // TODO: WriteConcern should be customizable in logger.properties
            this.collection.insert(dbo, WriteConcern.SAFE);

        } catch (Exception e) {
            reportError(e.getMessage(), e, ErrorManager.GENERIC_FAILURE);
        }
    }

    /**
     * Configure the log handler
     */
    protected void doConfigure() {
        this.host = getProperty(HOST, DEFAULT_HOST);
        this.port = getProperty(PORT, DEFAULT_PORT);
        this.collectionName = getProperty(COLLECTION, DEFAULT_COLLECTION);
        this.db = getProperty(DB, DEFAULT_DB);
        this.login = getProperty(LOGIN, null);
        this.password = getProperty(PASSWORD, null);
    }

    /**
     * Create all the required stuff to connect to MongoDB
     */
    protected void doConnect() throws UnknownHostException {
        doConfigure();
        List<ServerAddress> addresses = MongoDbHelper.getServerAddresses(this.host, this.port);

        this.mongo = MongoDbHelper.getMongo(addresses);
        this.mongoDB = getDatabase(this.mongo, this.db);

        if (login != null && login.trim().length() > 0) {
            if (!this.mongoDB.authenticate(login, password.toCharArray())) {
                throw new RuntimeException(
                        "Unable to authenticate with MongoDB server.");
            }
            // Allow password to be GCed
            password = null;
        }
        this.collection = this.mongoDB.getCollection(this.collectionName);
    }

    protected String getProperty(String name, String def) {
        final LogManager manager = LogManager.getLogManager();
        String result = manager.getProperty(this.getClass().getName() + "." + name);
        if (result == null) {
            result = def;
        }
        return result;
    }

    protected DB getDatabase(Mongo mongo, String databaseName) {
        return mongo.getDB(databaseName);
    }

    protected void closeDB() {
        if (this.mongo != null) {
            this.collection = null;
            this.mongo.close();
        }
    }
}
